import pymysql as mysql
from dh_agent.tasks import setup_job_context, jobs_running, workers_running, fix_file_permissions
from dh_agent import settings
from dh_corpus import *
from dh_job import *
import importlib
import time


def check_for_jobs():

    num_workers = workers_running()

    if num_workers < settings.MAX_AGENT_WORKERS:
        db = mysql.connect(
            host=settings.DB_HOST,
            user=settings.DB_USER,
            passwd=settings.DB_PASSWORD,
            db=settings.DB_SCHEMA
        )

        jobs_reserved = []
        jobs_launched = []

        with db:
            job_ids = Job(db).query_for_ids(status="Ready to run on job site", job_site_id=settings.JOB_SITE_ID)
            print("Found " + str(len(job_ids)) + " jobs to run...")

            job_site = JobSite(db, id=settings.JOB_SITE_ID)

            for ids in job_ids:
                if len(jobs_reserved) < settings.MAX_TASKS_PER_WORKER:
                    context = setup_job_context(connection=db, job_id=ids['id'], corpus_id=ids['corpus_id'])
                    context['job'].set_status("Job reserved")
                    jobs_reserved.append(context)
                else:
                    break
            db.commit()

            for context in jobs_reserved:
                for task_route in settings.TASKS:
                    if task_route['name'] == context['task'].name:
                        task = importlib.import_module(task_route['module'])
                        context['job'].set_status("Job running")
                        db.commit()
                        task.run(context)

                        if job_site.type == 'SLURM':
                            jobs_launched.append(context)
                        print("Launched job " + str(context['job'].id))

            del jobs_reserved[:] # clear for garbage collection

            while jobs_launched:
                # reset processes running in job contexts
                for x in range(0, len(jobs_launched)):
                    jobs_launched[x]['processes_running'] = 0

                running = jobs_running()

                for job in running:
                    process_signature = job['process_signature']

                    for x in range(0, len(jobs_launched)):
                        if jobs_launched[x]['job_signature'] in process_signature:
                            jobs_launched[x]['processes_running'] += 1

                for x in range(0, len(jobs_launched)):
                    if jobs_launched[x]['processes_running'] > 0:
                        jobs_launched[x]['job'].set_status("Job running: " + str(jobs_launched[x]['processes_running']) + " processes active")
                    else:
                        for task_route in settings.TASKS:
                            if task_route['name'] == context['task'].name:
                                if task_route.get('post_module', None):
                                    task = importlib.import_module(task_route['post_module'])
                                    task.run(jobs_launched[x])

                        fix_file_permissions(jobs_launched[x]['job_dir'])

                        destination_path = jobs_launched[x]['corpus'].kvp.get('job_results_path', jobs_launched[x]['corpus'].path)
                        destination_path += "/corpus/" + str(jobs_launched[x]['corpus'].id) + "/jobs/" + str(jobs_launched[x]['job'].id)


                        jft = JobFileTransfer(db)
                        jft.corpus_id = jobs_launched[x]['corpus'].id
                        jft.job_id = jobs_launched[x]['job'].id
                        jft.when = "post"
                        jft.src_path = jobs_launched[x]['job_dir']
                        jft.dest_path = destination_path
                        jft.recursive = True
                        jft.status = "ready for transfer"
                        jft.save()

                        jobs_launched[x]['job'].set_status("Job finished! Ready to transfer results")
                        jobs_launched.pop(x)
                        break

                db.commit()
                if jobs_launched:
                    time.sleep(settings.PROCESS_CHECK_DELAY)

        db.close()
