import logging
import yaml
import os

# LOGGING
logging.basicConfig(
    level=logging.DEBUG,
    format='%(asctime)s %(levelname)s %(message)s',
)

config_file = os.environ['DH_AGENT_CONFIG']
config = None
with open(config_file, 'r') as yaml_file:
    config = yaml.load(yaml_file)

BASE_PATH = config['dh_agent']['path']
JOB_SITE_ID = config['dh_agent']['job_site_id']
AGENT_USER = config['dh_agent']['user']
AGENT_GROUP = config['dh_agent']['group']

DB_HOST = config['dh_agent']['db_host']
DB_SCHEMA = config['dh_agent']['db_schema']
DB_USER = config['dh_agent']['db_user']
DB_PASSWORD = config['dh_agent']['db_password']
TASKS = config['tasks']

TESS3_TESSDATA_PREFIX = config['tesseract3']['tessdata_prefix']
TESS3_CONFIG = config['tesseract3']['config']

TESS4_TESSDATA_DIR = config['tesseract4']['tessdata_dir']
TESS4_PATH = config['tesseract4']['path']
TESS4_CONFIG = config['tesseract4']['config']

TESS_POSTPROC_DENOISING_PATH = config['tesseract_postprocessing']['denoising_path']
TESS_POSTPROC_SCRIPT_PATH = config['tesseract_postprocessing']['script_path']
TESS_POSTPROC_SCRIPT = config['tesseract_postprocessing']['script']
TESS_POSTPROC_MARIADB_BOOTSTRAP_SCRIPT = config['tesseract_postprocessing']['mariadb_bootstrap_script']
TESS_POSTPROC_MARIADB_START_SCRIPT = config['tesseract_postprocessing']['mariadb_start_script']
TESS_POSTPROC_SEASR_PATH = config['tesseract_postprocessing']['seasr_path']

OCULAR_MAX_HEAP = config['ocular']['max_heap']
OCULAR_ROOT = config['ocular']['path']
OCULAR_JAR = OCULAR_ROOT + "/ocular.jar"
OCULAR_UPDATE_ROOT = config['ocular']['update_path']

JUXTA_PATH = config['juxta']['path']
JUXTA_ALGORITHM = config['juxta']['algorithm']
JUXTA_HYPHEN = config['juxta']['hyphen']

# SET GOOGLE SERVICE ACCOUNT JSON
os.environ['GOOGLE_APPLICATION_CREDENTIALS'] = config['google']['service_account_json']
GOOGLE_CLOUD_VISION_API_KEY = config['google']['api_key']

SLURM_MEMORY_PER_CPU = int(config['slurm']['memory_per_cpu'])
SLURM_CPUS_PER_TASK = int(config['slurm']['cpus_per_task'])
SLURM_WALLTIME = int(config['slurm']['walltime'])
SLURM_QUEUE = config['slurm']['queue']
SLURM_MAX_JOBS = int(config['slurm']['max_jobs'])

# number of seconds to wait in between job process checks
PROCESS_CHECK_DELAY = int(config['slurm']['process_check_delay'])

MAX_AGENT_WORKERS = int(config['dh_agent']['max_workers'])
MAX_TASKS_PER_WORKER = int(config['dh_agent']['max_tasks_per_worker'])
