import json
import os
from subprocess import Popen, PIPE, call
from dh_agent import settings
from dh_corpus import *
from dh_job import *


def setup_job_context(connection, job_id, corpus_id):

    corpus = Corpus(connection, id=corpus_id)
    job = Job(connection, id=job_id, corpus_id=corpus_id)
    config = json.loads(job.configuration)
    job_site = JobSite(connection, id=job.job_site_id)
    task = Task(connection, id=job.task_id)
    if job_site.transfer_type == "local":
        site_dir = corpus.kvp.get('job_results_path', corpus.path)
    else:
        site_dir = job_site.job_dir + corpus.kvp.get('job_dir_suffix', '')
    corpus_dir = site_dir + "/corpus"
    jobs_dir = corpus_dir + "/jobs"
    job_dir = site_dir + "/corpus/" + str(corpus_id) + "/jobs/" + str(job.id)
    slurm_dir = job_dir + '/SLURM'
    log_dir = job_dir + '/log'
    job_signature = 't' + str(task.id) + 'j' + str(job.id)

    if not os.path.exists(corpus_dir):
        os.makedirs(corpus_dir)
        fix_file_permissions(corpus_dir)

    if not os.path.exists(jobs_dir):
        os.makedirs(jobs_dir)
        fix_file_permissions(jobs_dir)

    # Make Job, SLURM, log directories
    if not os.path.exists(job_dir):
        os.makedirs(job_dir)

    if job_site.type == 'SLURM' and not os.path.exists(slurm_dir):
        os.makedirs(slurm_dir)

    if not os.path.exists(log_dir):
        os.makedirs(log_dir)

    fix_file_permissions(job_dir)

    return {
        'corpus': corpus,
        'job': job,
        'job_signature': job_signature,
        'config': config,
        'job_site': job_site,
        'task': task,
        'job_dir': job_dir,
        'site_dir': site_dir,
        'slurm_dir': slurm_dir,
        'log_dir': log_dir,
        'connection': connection,
        'processes_running': 0
    }


def fix_file_permissions(path):
    fixed = False

    if os.path.exists(path):
        command = [
            'chmod', '-R',
            '775',
            path
        ]

        return_code = call(command)

        if return_code == 0:
            command = [
                'chown', '-R',
                '{0}:{1}'.format(settings.AGENT_USER, settings.AGENT_GROUP),
                path
            ]

            return_code = call(command)

            if return_code == 0:
                fixed = True

    return fixed


def get_filename(path):
    path_parts = path.split('/')
    return path_parts[len(path_parts) - 1]


def get_file_extension(path):
    path_parts = path.split('.')
    return "." + path_parts[len(path_parts) - 1]


def replace_last_instance(string, old, new):
    index = string.rfind(old)
    return string[:index] + new + string[index + len(old):]


def jobs_running():
    command = Popen(
        [
            "squeue",
            "-a",
            "-h",
            "-u", settings.AGENT_USER
        ], stdout=PIPE
    )
    lines = command.stdout.readlines()

    running = []

    for byte_line in lines:
        line = byte_line.decode("utf-8")
        parts = line.strip().split(' ')
        relevant_parts = []
        for part in parts:
            if part:
                relevant_parts.append(part)

        if len(relevant_parts) == 8:
            running.append(
                {
                    'process_id': relevant_parts[0],
                    'process_signature': relevant_parts[2],
                    'time_running': relevant_parts[5]
                }
            )

    return running


def workers_running():
    command = Popen(
        [
            "ps",
            "aux"
        ], stdout=PIPE
    )
    lines = command.stdout.readlines()

    num_workers = 0

    for byte_line in lines:
        line = byte_line.decode("utf-8")
        parts = line.strip().split(' ')
        relevant_parts = []
        for part in parts:
            if part:
                relevant_parts.append(part)

        for part in relevant_parts:
            if "dh_agent/cron.py" in part:
                num_workers += 1

    return num_workers
