import requests
import json
import re


def run(context):

    document_url = requests.get(context['config']['parameters']['document_url']['value'])
    document_html = document_url.text

    dublin_core_pattern = r'<meta name=\"(DC\.[^\"]*)\" content=\"([^\"]*)\" />'
    line_break_pattern = '<hr />'

    document_html_path = context['job_dir'] + '/document.html'
    document_import_metadata_path = context['job_dir'] + '/document_import.json'
    page_html_path_pattern = context['job_dir'] + '/page_{0}.html'

    document_import_metadata = {
        'title': None,
        'author': None,
        'html_import_file': 'document.html',
        'kvp': {},
        'pages': []
    }

    if document_html:
        # write out document HTML file
        with open(document_html_path, 'w') as file:
            file.write(document_html)

        # extract dublin core metadata
        lines = document_html.splitlines()
        for line in lines:
            match = re.search(dublin_core_pattern, line)
            if match:
                document_import_metadata['kvp'][match.group(1)] = match.group(2)

        # try to assign title and author based off of extracted dublin core metadata
        document_import_metadata['title'] = document_import_metadata['kvp'].get('DC.title', None)
        document_import_metadata['author'] = document_import_metadata['kvp'].get('DC.creator', None)

        # make document pages and write page HTML files
        pages_html = re.split(line_break_pattern, document_html)
        for page_index in range(1, len(pages_html)):
            page_number = page_index + 1
            page_html_path = page_html_path_pattern.format(page_number)
            with open(page_html_path, 'w') as file:
                file.write(pages_html[page_index])
            document_import_metadata['pages'].append({
                'ref_no': page_number,
                'html_import_file': 'page_{0}.html'.format(page_number)
            })


        # write out document_import_metadata
        with open(document_import_metadata_path, 'w') as file:
            json.dump(document_import_metadata, file)

    context['job'].set_status("Job finished! Ready to transfer results")