import os
import requests
from dh_agent import settings
from dh_agent.tasks import get_file_extension
from dh_corpus import *
from dh_job import *
from PIL import Image
import base64
import json
from google.cloud import vision


def run(context):

    pageset = PageSet(context['connection'], id=context['job'].target_id, corpus_id=context['job'].corpus_id)
    pages = pageset.all_pages

    client = vision.Client()

    for page in pages:

        if os.path.exists(page.path):

            image_path = page.path
            ext = get_file_extension(image_path)
            if ext.lower() == '.tif' or ext.lower() == '.tiff':
                new_image_path = context['job_dir'] + '/' + os.path.basename(image_path).replace(ext, '.jpg')
                im = Image.open(image_path)
                im.save(new_image_path)
                image_path = new_image_path
                ext = '.jpg'

            # GET PLAIN TEXT USING GC PYTHON API
            text_path = context['job_dir'] + '/' + os.path.basename(image_path).replace(ext, '.txt')
            with open(image_path, 'rb') as image_file:
                image = client.image(content=image_file.read())
            text = image.detect_full_text().text
            with open(text_path, 'w') as text_file:
                text_file.write(text)

            # GET JSON USING GCV REST API
            json_path = context['job_dir'] + '/' + os.path.basename(image_path).replace(ext, '.json')
            with open(image_path, 'rb') as image_file:
                image_content = base64.b64encode(image_file.read()).decode('UTF-8')
            request_json = {
                "requests": [
                    {
                        "image": {
                            "content": image_content
                        },
                        "features": [
                            {
                                "type": "TEXT_DETECTION",
                                "maxResults": 10000
                            }
                        ]
                    }
                ]
            }
            response = requests.post(
                url='https://vision.googleapis.com/v1/images:annotate?key=' + settings.GOOGLE_CLOUD_VISION_API_KEY,
                data=json.dumps(request_json),
                headers={
                    'Content-Type': 'application/json'
                }
            )
            with open(json_path, 'w') as json_out:
                json_out.write(response.text)

    context['job'].set_status("Job finished! Ready to transfer results")
