import os
from PyPDF2 import PdfFileReader
from subprocess import call
from dh_agent import settings
from dh_corpus import *
from dh_agent.tasks import get_file_extension


def run(context):

    pdf_file = context['site_dir'] + context['config']['parameters']['pdf_file']['value']
    density_value = context['config']['parameters']['image_dpi']['value']
    split_images = context['config']['parameters']['split_images']['value'] == 'Yes'

    if os.path.exists(pdf_file):
        num_pages = None
        with open(pdf_file, 'rb') as pdf_in:
            pdf = PdfFileReader(pdf_in)
            num_pages = pdf.getNumPages()

        if num_pages:
            slurm_files = []
            slurm_file_path = ""
            slurm_file = None
            proc_num = 1

            file_name = os.path.basename(pdf_file)
            file_name = file_name.replace(get_file_extension(file_name), "")

            for page_num in range(0, num_pages):
                if not slurm_file or page_num % PAGES_PER_TASK == 0:
                    if slurm_file:
                        slurm_files.append(slurm_file_path)
                        slurm_file.close()
                        proc_num += 1

                    slurm_file_path = context['slurm_dir'] + '/process_' + str(proc_num) + '.slrm'
                    slurm_file = open(slurm_file_path, 'w')
                    slurm_file.write(SLURM_PREFIX)

                page_suffix = str(page_num + 1)
                while len(page_suffix) < 4:
                    page_suffix = "0" + page_suffix

                command = [
                    "convert",
                    "-density", density_value,
                    pdf_file + "[" + str(page_num) + "]"
                ]

                if split_images:
                    command.append("-crop")
                    command.append("50%x100%")
                    command.append("+repage")

                command.append(context['job_dir'] + '/' + file_name + "_" + page_suffix + ".png")

                slurm_file.write(" ".join(command) + "\n")

            if slurm_file:
                slurm_files.append(slurm_file_path)
                slurm_file.close()

            for proc_num in range(0, len(slurm_files)):

                proc_signature = context['job_signature'] + "p" + str(proc_num + 1)

                command = [
                    "sbatch", "--parsable",
                    "-p", settings.SLURM_QUEUE,
                    "-J", proc_signature,
                    "-o", context['log_dir'] + '/' + proc_signature + '.log',
                    "--mem-per-cpu", str(MEMORY_PER_CPU),
                    "--cpus-per-task", str(CPUS_PER_TASK),
                    "--time", "60",
                    slurm_files[proc_num]
                ]

                call(command)

            context['job'].set_status('Job running: ' + str(len(slurm_files)) + ' processes launched')

SLURM_PREFIX = '''#!/bin/bash
#SBATCH --nodes=1
#SBATCH --ntasks=1
#SBATCH --signal=USR1@300

'''

MEMORY_PER_CPU = 4000
CPUS_PER_TASK = 1
PAGES_PER_TASK = 10