import json
import os
from dh_agent import settings
from dh_corpus import *
from dh_job import *
from dh_agent.tasks import get_filename
from subprocess import call


def run(context):

    doclist_path = context['job_dir'] + "/document_list.txt"

    document = Document(context['connection'], id=context['job'].target_id, corpus_id=context['job'].corpus_id)
    pageset = PageSet(context['connection'], id=document.kvp["training_set"], corpus_id=document.corpus_id)
    pages = pageset.all_pages

    # Make DocList
    with open(doclist_path, "w") as fout:
        for page in pages:
            fout.write(context['site_dir'] + page.path + "\n")

    command = [
        "java", settings.OCULAR_MAX_HEAP,
        "-Done-jar.main.class=edu.berkeley.cs.nlp.ocular.main.TrainFont",
        "-jar", settings.OCULAR_JAR,
        "-outputPath", context['job_dir'],
        "-inputDocListPath", doclist_path,
        "-inputFontPath", context['site_dir'] + context['config']['parameters']['input_font_path']['value'],
        "-inputLmPath", context['site_dir'] + context['config']['parameters']['input_language_model_path']['value'],
        "-inputGsmPath", context['site_dir'] + context['config']['parameters']['input_gsm_path']['value'],
        "-outputFontPath", context['job_dir'] + "/" + get_filename(context['config']['parameters']['input_font_path']['value']),
        "-outputLmPath", context['job_dir'] + "/" + get_filename(context['config']['parameters']['input_language_model_path']['value']),
        "-outputGsmPath", context['job_dir'] + "/" + get_filename(context['config']['parameters']['input_gsm_path']['value']),
        "-continueFromLastCompleteIteration", context['config']['parameters']['continue_from_last_complete_iteration']['value'],
        "-allowGlyphSubstitution", context['config']['parameters']['allow_glyph_substitution']['value'],
        "-updateLM", context['config']['parameters']['update_lm']['value'],
        "-updateGsm", context['config']['parameters']['update_gsm']['value'],
        "-emissionEngine", context['config']['parameters']['emission_engine']['value'],
        "-binarizeThreshold", context['config']['parameters']['binarizeThreshold']['value'],
        "-allowOverlappingLines", context['config']['parameters']['allow_overlapping_lines']['value'],
        "-crop", context['config']['parameters']['crop']['value']
    ]

    command_string = " ".join(command)
    proc_signature = context['job_signature'] + 'p1'
    slurm_file_path = context['slurm_dir'] + '/' + proc_signature + '.slrm'

    slurm_file = SLURM_PREFIX
    slurm_file += "export OCULAR_ROOT='" + settings.OCULAR_ROOT + "'\n"
    slurm_file += "export OCULAR='" + settings.OCULAR_JAR + "'\n\n"
    slurm_file += command_string + "\n"

    with open(slurm_file_path, 'w') as fout:
        fout.write(slurm_file)

    command = [
        "sbatch", "--parsable",
        "-p", settings.SLURM_QUEUE,
        "-J", proc_signature,
        "-o", context['log_dir'] + '/' + proc_signature + '.log',
        "--mem-per-cpu", str(settings.SLURM_MEMORY_PER_CPU),
        "--cpus-per-task", str(SLURM_CPUS_PER_TASK),
        "--time", str(MINUTES_PER_PAGE * len(pages)),
        slurm_file_path
    ]

    call(command)



SLURM_PREFIX = '''#!/bin/bash
#SBATCH --nodes=1
#SBATCH --ntasks=1
#SBATCH --signal=USR1@300

module load gcc/5.2.0
module load java/1.8.0_77
module load amd-opencl/3.0
'''

MINUTES_PER_PAGE = 60

SLURM_CPUS_PER_TASK = 8


'''
module load gcc/4.8.2
module load java/1.7.0_67
module load amd-opencl/3.0
'''
