import json
import os
import glob
from dh_agent import settings
from dh_corpus import *
from dh_job import *
from dh_agent.tasks import get_filename
from subprocess import call


def run(context):

    processes = []
    proc_counter = 0
    page_counter = 0

    document = Document(context['connection'], id=context['job'].target_id, corpus_id=context['job'].corpus_id)
    pages = document.all_pages

    font_path = ''
    lm_path = ''
    gsm_path = ''

    # get font_path, lm_path, and gsm_path from previous job selected by user
    prev_job_id = context['config']['parameters']['input_training_job']['value']
    prev_job_dir = context['site_dir'] + "/corpus/" + str(document.corpus.id) + "/jobs/" + prev_job_id
    if os.path.exists(prev_job_dir):
        prev_job_files = filter(os.path.isfile, glob.glob(prev_job_dir + '/*'))
        for prev_job_file in prev_job_files:
            if '.fontser' in prev_job_file:
                font_path = prev_job_file
            elif '.lmser' in prev_job_file:
                lm_path = prev_job_file
            elif '.gsmser' in prev_job_file:
                gsm_path = prev_job_file
            elif font_path and lm_path and gsm_path:
                break

    if font_path and lm_path and gsm_path:

        while proc_counter < context['job_site'].max_jobs and page_counter < len(pages):
            proc_counter += 1
            processes.append(
                {
                    'process_signature': context['job_signature'] + 'p' + str(proc_counter),
                    'pages': [context['site_dir'] + pages[page_counter].path]
                }
            )
            page_counter += 1

        while page_counter < len(pages):
            if not proc_counter < len(processes):
                proc_counter = 0

            processes[proc_counter]['pages'].append(context['site_dir'] + pages[page_counter].path)
            page_counter += 1
            proc_counter += 1

        # Make DocLists
        for process in processes:
            doclist_path = context['job_dir'] + '/' + process['process_signature'] + '_document_list.txt'
            with open(doclist_path, "w") as fout:
                for path in process['pages']:
                    fout.write(path + "\n")
            process['doclist_path'] = doclist_path

        with open(context['job_dir'] + '/processes.json', 'w') as fout:
            json.dump(processes, fout)

        for process in processes:

            command = [
                "java", settings.OCULAR_MAX_HEAP,
                "-Done-jar.main.class=edu.berkeley.cs.nlp.ocular.main.Transcribe",
                "-jar", settings.OCULAR_JAR,
                "-outputPath", context['job_dir'],
                "-inputDocListPath", process['doclist_path'],
                "-inputFontPath", font_path,
                "-inputLmPath", lm_path,
                "-inputGsmPath", gsm_path,
                "-allowGlyphSubstitution", context['config']['parameters']['allow_glyph_substitution']['value'],
                "-allowOverlappingLines", context['config']['parameters']['allow_overlapping_lines']['value'],
                "-updateLM", "false",
                "-updateGsm", "false",
                "-emissionEngine", context['config']['parameters']['emission_engine']['value'],
                "-outputFormats", "dipl,norm,alto"
            ]

            command_string = " ".join(command)
            slurm_file_path = context['slurm_dir'] + '/' + process['process_signature'] + '.slrm'

            slurm_file = SLURM_PREFIX
            slurm_file += "export OCULAR_ROOT='" + settings.OCULAR_ROOT + "'\n"
            slurm_file += "export OCULAR='" + settings.OCULAR_JAR + "'\n\n"
            slurm_file += command_string + "\n"

            with open(slurm_file_path, 'w') as fout:
                fout.write(slurm_file)


            command = [
                "sbatch", "--parsable",
                "-p", settings.SLURM_QUEUE,
                "-J", process['process_signature'],
                "-o", context['log_dir'] + '/' + process['process_signature'] + '.log',
                "--mem-per-cpu", str(settings.SLURM_MEMORY_PER_CPU),
                "--cpus-per-task", str(SLURM_CPUS_PER_TASK),
                "--time", str(MINUTES_PER_PAGE * len(process['pages'])),
                slurm_file_path
            ]

            call(command)



SLURM_PREFIX = '''#!/bin/bash
#SBATCH --nodes=1
#SBATCH --ntasks=1
#SBATCH --signal=USR1@300

module load gcc/5.2.0
module load java/1.8.0_77
module load amd-opencl/3.0
'''

MINUTES_PER_PAGE = 60

SLURM_CPUS_PER_TASK = 8


'''
module load gcc/4.8.2
module load java/1.7.0_67
module load amd-opencl/3.0
'''
