from subprocess import call
from dh_agent import settings
from dh_corpus import *
from dh_agent.tasks import get_filename, get_file_extension, replace_last_instance


def run(context):

    document = Document(context['connection'], id=context['job'].target_id, corpus_id=context['job'].corpus_id)
    pages = document.all_pages
    total_pages = len(pages)
    pages_per_job = 1

    # Determine number of pages per SLURM job
    if total_pages > context['job_site'].max_jobs:
        pages_per_job = total_pages / context['job_site'].max_jobs

    if pages_per_job < 1:
        pages_per_job = 1

    # Write tesseract config file
    tesseract_config_file = context['job_dir'] + "/tesseract.config"
    with open(tesseract_config_file, "w") as fout:
        fout.write(TESSERACT_CONFIG)

    proc = 1
    proc_page_count = 0
    slurm_file = None
    slurm_files = {}

    for page in pages:
        if not slurm_file:
            slurm_file_path = context['slurm_dir'] + '/process_' + str(proc) + '.slrm'
            slurm_file = open(slurm_file_path, 'w')
            slurm_file.write(SLURM_PREFIX)
            slurm_files[str(proc)] = slurm_file_path

        output_file = get_filename(page.path)
        extension = get_file_extension(output_file)
        output_file = replace_last_instance(output_file, extension, '')

        command = [
            TESSERACT_PATH,
            context['site_dir'] + page.path,
            context['job_dir'] + '/' + output_file,
            "--tessdata-dir", TESSDATA_DIR,
            "-l", context['config']['parameters']['language']['value'],
            tesseract_config_file
        ]

        slurm_file.write(" ".join(command) + "\n")
        proc_page_count += 1

        if proc_page_count >= pages_per_job:
            slurm_file.close()
            slurm_file = None
            proc_page_count = 0
            proc += 1

    if slurm_file:
        slurm_file.close()

    for proc, file in slurm_files.items():
        proc_signature = context['job_signature'] + "p" + proc

        command = [
            "sbatch", "--parsable",
            "-p", settings.SLURM_QUEUE,
            "-J", proc_signature,
            "-o", context['log_dir'] + '/' + proc_signature + '.log',
            "--mem-per-cpu", str(MEMORY_PER_CPU),
            "--cpus-per-task", str(CPUS_PER_TASK),
            "--time", str(int(MINUTES_PER_PAGE * pages_per_job)),
            file
        ]

        call(command)

    context['job'].set_status('Job running: ' + str(len(slurm_files)) + ' processes launched')


SLURM_PREFIX = '''#!/bin/bash
#SBATCH --nodes=1
#SBATCH --ntasks=1
#SBATCH --signal=USR1@300

module load gcc/5.2.0
module load leptonica

'''

MEMORY_PER_CPU = 4000
CPUS_PER_TASK = 2

TESSDATA_DIR = settings.TESS4_TESSDATA_DIR
TESSERACT_PATH = settings.TESS4_PATH
TESSERACT_CONFIG = settings.TESS4_CONFIG
MINUTES_PER_PAGE = 3