import os
import glob
import time
from subprocess import call
from dh_agent import settings
from dh_corpus import *
from dh_agent.tasks import get_filename, get_file_extension, replace_last_instance


def run(context):

    document = Document(context['connection'], id=context['job'].target_id, corpus_id=context['job'].corpus_id)
    pages = document.all_pages
    prev_job_files = []

    # get font_path, lm_path, and gsm_path from previous job selected by user
    prev_job_id = context['config']['parameters']['input_ocr_job']['value']
    prev_job_dir = context['site_dir'] + "/corpus/" + str(document.corpus.id) + "/jobs/" + prev_job_id
    if os.path.exists(prev_job_dir):
        prev_job_files = list(filter(os.path.isfile, glob.glob(prev_job_dir + '/*')))

    total_pages = len(pages)
    pages_per_job = PAGES_PER_TASK

    # Determine number of pages per SLURM job
    #if total_pages > context['job_site'].max_jobs:
    #    pages_per_job = total_pages / context['job_site'].max_jobs

    if pages_per_job < 1:
        pages_per_job = 1

    proc = 1
    proc_page_count = 0
    slurm_file = None
    slurm_files = {}

    for page in pages:
        if not slurm_file:
            slurm_file_path = context['slurm_dir'] + '/process_' + str(proc) + '.slrm'
            slurm_file = open(slurm_file_path, 'w')
            slurm_file.write(SLURM_PREFIX)
            slurm_file.write(MARIADB_BOOTSTRAP_SCRIPT + "\n")
            slurm_file.write(MARIADB_START_SCRIPT + "\n\n")
            slurm_file.write("cd " + POSTPROC_SCRIPT_PATH + "\n")
            slurm_files[str(proc)] = slurm_file_path

        hocr_file_path = ""
        ground_truth_file = context['site_dir'] + page.kvp.get("ground_truth_file", "")

        for file_path in prev_job_files:
            if get_filename(file_path).replace(get_file_extension(file_path), '') in get_filename(page.path) and get_file_extension(file_path) == '.xml':
                hocr_file_path = file_path
                break

        if hocr_file_path and ground_truth_file:

            command = [
                "python",
                POSTPROC_SCRIPT,
                hocr_file_path,
                context['job_dir'],
                SEASR_PATH,
                "${TMPDIR}/emop.properties",
                JUXTA_PATH,
                ground_truth_file
            ]

            slurm_file.write(" ".join(command) + "\n")

            proc_page_count += 1

            if proc_page_count >= pages_per_job:
                slurm_file.close()
                slurm_file = None
                proc_page_count = 0
                proc += 1

    if slurm_file:
        slurm_file.close()

    for proc, file in slurm_files.items():
        proc_signature = context['job_signature'] + "p" + proc

        command = [
            "sbatch", "--parsable",
            "-p", settings.SLURM_QUEUE,
            "-J", proc_signature,
            "-o", context['log_dir'] + '/' + proc_signature + '.log',
            "--mem-per-cpu", str(MEMORY_PER_CPU),
            "--cpus-per-task", str(CPUS_PER_TASK),
            "--time", str(int(MINUTES_PER_PAGE * pages_per_job)),
            file
        ]

        call(command)
        time.sleep(SLURM_WAIT_TIME)

    context['job'].set_status('Job running: ' + str(len(slurm_files)) + ' processes launched')


SLURM_PREFIX = '''#!/bin/bash
#SBATCH --nodes=1
#SBATCH --ntasks=1
#SBATCH --signal=USR1@300

module load gcc/4.8.2
module load python/2.7.10
module load java/1.7.0_67
module load mariadb/10.0.15

'''

MEMORY_PER_CPU = 4000
CPUS_PER_TASK = 1

POSTPROC_SCRIPT_PATH = settings.TESS_POSTPROC_SCRIPT_PATH
POSTPROC_SCRIPT = settings.TESS_POSTPROC_SCRIPT
MARIADB_BOOTSTRAP_SCRIPT = settings.TESS_POSTPROC_MARIADB_BOOTSTRAP_SCRIPT
MARIADB_START_SCRIPT = settings.TESS_POSTPROC_MARIADB_START_SCRIPT
JUXTA_PATH = settings.JUXTA_PATH
SEASR_PATH = settings.TESS_POSTPROC_SEASR_PATH
SLURM_WAIT_TIME = 4
MINUTES_PER_PAGE = 3
PAGES_PER_TASK = 5